package Sys::Info::Driver::Linux::Device;
$Sys::Info::Driver::Linux::Device::VERSION = '0.7907';
use strict;
use warnings;

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Sys::Info::Driver::Linux::Device

=head1 VERSION

version 0.7907

=head1 SYNOPSIS

    use parent qw( Sys::Info::Driver::Linux::Device );

=head1 DESCRIPTION

Base class for Linux device drivers.

=head1 NAME

Sys::Info::Driver::Linux::Device - Base class for Linux device drivers

=head1 METHODS

None.

=head1 AUTHOR

Burak Gursoy

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2006 by Burak Gursoy.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
